<?php

/******************************************************************************
*
*  File:  backup_db.php
*  --------------------
*
*  Purpose:   This script creates a backup file of any MySQL database, regard-
*             less of its design (all tables, all fields). The design of the 
*             file is adjusted to the script language of MySQL, so that it can 
*             be processed as a source-file (see MySQL command "SOURCE"). 
*             All tables will be dropped at first and rebuilded again and 
*             filled up with all data fields.
*             This script is able to run in batch mode. For this the name
*             of the written file is given automatically in this pattern:
*             "BU-YYYYMMDD-HHIISS.sql" (year, month, day, hour, min, sec)
*             and stored in a given directory.
*
*  Usage:     Create a directory, where you want to store this script.
*             For example: "backup". Safe this script there. For safety
*             reasons it is recommended, that you should protect this
*             directory by a password using the ".htaccess"/".htpasswd" file.
*             This directory must not lay deeper than one level down of the
*             phpbb-root. Otherwise some more corrections must be done in this
*             script.
*
*             Don't forget to fill the parameters at the head of the source!
*
*  Written:   2007-01-29
*
*  Copyright: (C) 2007 Ron
*
*******************************************************************************/

// ******************************* Please edit !
$backup_dir = "../archive/";  // Name of archive dir, will be created if not exist
$phpbb_root_path = '../';
// *********************************************


define('IN_PHPBB', true);

include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'config.'.$phpEx);

$startdate = date("Ymd-His");

$backup_file = $backup_dir . "BU-" . $startdate . ".sql";

global $handle;
global $anz_tabellen;
global $anz_felder;
global $anz_saetze;


// ---------------------------------------------------------------------
// This function creates a table of all tables of the database.
// ---------------------------------------------------------------------

function get_table_of_tables()
{
	global $anz_tabellen;
	$tabelle = array();
	$table_query = "SHOW TABLES";
	$result = mysql_query($table_query);
	if(!$result) {
		echo "***** Fehler bei Zugriff auf db<br>";
	}
      else {
		$i = 0;
		while ($zeile = mysql_fetch_row ($result)) {
			$tabelle [$i] = $zeile[0];
			$i += 1;
		}
		$anz_tabellen += $i;
		mysql_free_result($result);
		return $tabelle;
	}
} // End get_table_of_tables


// ---------------------------------------------------------------------
// This function creates the definition of a table.
// ---------------------------------------------------------------------

function get_table_def($table)
{
	global $anz_felder;
	global $anz_schluessel;

	$schema_create = "";
	$field_query = "SHOW FIELDS FROM $table";
	$key_query = "SHOW KEYS FROM $table";

      $schema_create .= "DROP TABLE IF EXISTS $table;\n";
	
	$schema_create .= "CREATE TABLE $table(\n";

	$result = mysql_query($field_query);
	if(!$result) {
		echo "***** Fehler bei show fields: " . $field_query . "<br>";
	}

	while ($row = mysql_fetch_assoc($result)) {

		$anz_felder++;

		$schema_create .= '	' . $row['Field'] . ' ' . $row['Type'];

		if(!empty($row['Default'])) {
			$schema_create .= ' DEFAULT \'' . $row['Default'] . '\'';
		}

		if($row['Null'] != "YES") {
			$schema_create .= ' NOT NULL';
		}

		if($row['Extra'] != "") {
			$schema_create .= ' ' . $row['Extra'];
		}

		$schema_create .= ",\n";
	}
	$schema_create = ereg_replace(',' . "\n" . '$', "", $schema_create);

	// find keyfiels
 
	$result = mysql_query($key_query);
	if(!$result) {
		echo "***** Fehler in get_table_def (show keys)" . $key_query . "<br>";
	}

	while($row = mysql_fetch_assoc($result)) {

		$anz_schluessel++;

		$kname = $row['Key_name'];

		if(($kname != 'PRIMARY') && ($row['Non_unique'] == 0)) {
			$kname = "UNIQUE|$kname";
		}

		if(!is_array($index[$kname]))	{
			$index[$kname] = array();
		}

		$index[$kname][] = $row['Column_name'];
	}

	while(list($x, $columns) = @each($index))	{

		$schema_create .= ", \n";

		if($x == 'PRIMARY'){
			$schema_create .= '	PRIMARY KEY (' . implode($columns, ', ') . ')';
		}
		elseif (substr($x,0,6) == 'UNIQUE')	{
			$schema_create .= '	UNIQUE ' . substr($x,7) . ' (' . implode($columns, ', ') . ')';
		}
		else {
			$schema_create .= "	KEY $x (" . implode($columns, ', ') . ')';
		}
	}

	$schema_create .= "\n);";

	if(get_magic_quotes_runtime()) {
		return(stripslashes($schema_create));
	}
	else {
		return($schema_create);
	}

} // End get_table_def


// ---------------------------------------------------------------------
// This function reads the data fiels.
// ---------------------------------------------------------------------

function get_table_content($table)
{
	global $handle;
	global $anz_saetze;

	$field_names = array();

	// Grab the list of field names.

	$field_query = "SHOW FIELDS FROM $table";

	$result = mysql_query($field_query);
	if(!$result) {
		echo "***** Fehler bei show fields: " . $field_query . "<br>";
	}
	
	$i=0;
	while ($row = mysql_fetch_assoc($result)) {
		$field_names [$i] = $row['Field'];
		$i++;
	}

	$num_fields = $i;

	// Grab the data from the table.

	if (!($result = mysql_query("SELECT * FROM $table"))) {
		echo "***** Fehler bei get_table_content (SELECT * FROM $table)<br>";
	}

	// Loop through the resulting rows and build the sql statement.

	if ($row = mysql_fetch_assoc($result)) {

		fwrite ($handle, "\n#\n# Table Data for $table\n#\n");
	
		$table_list = '(';

		for ($j = 0; $j < $num_fields; $j++)
		{
			$table_list .= (($j > 0) ? ', ' : '') . $field_names[$j];
		}
		$table_list .= ')';

		do
		{
			// Start building the SQL statement.

			$anz_saetze++;

			$schema_insert = "INSERT INTO $table $table_list VALUES(";

			// Loop through the rows and fill in data for each column
			for ($j = 0; $j < $num_fields; $j++)
			{
				$schema_insert .= ($j > 0) ? ', ' : '';

				if(!isset($row[$field_names[$j]]))
				{
					$schema_insert .= 'NULL';
				}
				elseif ($row[$field_names[$j]] != '')
				{
					$schema_insert .= '\'' . addslashes($row[$field_names[$j]]) . '\'';
				}
				else
				{
					$schema_insert .= '\'\'';
				}
			}

			$schema_insert .= ');';

			fwrite ($handle, (trim($schema_insert)) . "\n");

		}
		while ($row = mysql_fetch_assoc($result));
	}

	return(true);

} // End of get_table_content_mysql

function output_table_content($content)
{
      global $handle;
	fwrite ($handle, $content . "\n");
	return;
}

function open_db()
{
	global $db;
	global $dbhost;
	global $dbuser;
	global $dbpasswd;
	global $dbname;

	$db = mysql_connect($dbhost, $dbuser, $dbpasswd);
	mysql_select_db($dbname);
}

function close_db()
{
	global $db;
      mysql_close($db);
}

function make_dir ($s)
{
  if (file_exists($s)) {
     if (!is_dir($s)) {
       die("***** File with same name as archivedir already exists ($s) <br>");
     }
  }
  else {
     if (mkdir ($s)) {
        echo "***** New archive directory created ($s) <br>";
     }
     else {
        die ("***** Cannot create new archive directory ($s) <br>");
     }
  }
}

function file_open()
{
	global $handle;
	global $backup_file;
	if (!$handle = fopen ($backup_file, "w")){
		echo "***** Fehler bei ffnen der Datei ($backup_file)! <br>";
	}
}

function file_close()
{
	global $handle;
	if (!fclose ($handle)) {
		echo "***** Fehler bei Schlieen der Datei! <br>";
	}
}

//
// End Functions
// -------------


//
// Begin program proper ******************************************************
//

	echo "***** start backup_db at $startdate --- (c) 2007 by Ron<br>";

    make_dir($backup_dir);
	file_open();
	open_db();

	$anz_tabellen = 0;
	$anz_saetze = 0;
	$anz_felder = 0;
	$anz_schluessel = 0;

	$tables = get_table_of_tables();

	fwrite ($handle, "#\n");
	fwrite ($handle, "# Backup Script created by backup_db.php (c) 2007 by Ron\n");
	fwrite ($handle, "# Dump of tables for $db_name\n");
	fwrite ($handle, "#\n# DATE : " .  gmdate("d-m-Y H:i:s", time()) . " GMT\n");
	fwrite ($handle, "#\n");

	for($i = 0; $i < count($tables); $i++){

		$table_name = $tables[$i];

		$table_def_function = "get_table_def";
		$table_content_function = "get_table_content";

		fwrite ($handle, "#\n# TABLE: " . $table_name . "\n#\n");
		fwrite ($handle, $table_def_function($table_name) . "\n");
		$table_content_function($table_name);
	}

	close_db();
	file_close();
	echo "***** SQL file: " . $backup_file . " written.<br>";
	echo "***** number of tables = " . $anz_tabellen . "<br>";
	echo "***** number of fields = " . $anz_felder . "<br>";
	echo "***** number of keys = " . $anz_schluessel . "<br>";
	echo "***** number of data records = " . $anz_saetze . "<br>";
	echo "***** stop backup_db<br>";

?>
