<?php

/******************************************************************************
*
*  File:  restore_db.php
*  ---------------------
*
*  Purpose:   This script creates a list of a given archive directory with all
*             its files and for each filename a button to start a restore with
*             MySQL or a download of that file via browser.
*             For security reasons you should store this script in a subdirec-
*             tory, which is password protected (see .htaccess/.htpasswd file). 
*             This is the only way, that nobody else can run this script.
*             Restoring a SQL file can only be done with MySQL software. If
*             you will run another database maintenance system you have to
*             change the corresponding database commands manually.
*             Don't forget to fill the parameters at the head of the source.
*
*  Written:   2007/09/30
*
*  Copyright: (C) 2007 by Ron
*
*******************************************************************************/

// ----------------------------------------------------------- fill variables
$backupdir = "../archive";
$phpbb_root_path = '../';
// --------------------------------------------------------------------------
// ------------------------------ system variables for long runtime !
// ------------------------------ these parameters may not effective, if
// ------------------------------ the server is switched to SAFE ON mode !
// ------------------------------ in this case the script will be stopped
// ------------------------------ if maximum time is exceeded.
// --------------------------------------------------------------------------
ini_set("memory_limit","200M");  // max memory 200 MB
set_time_limit(14400);           // max time 4 hours
// --------------------------------------------------------------------------

define('IN_PHPBB', true);

include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'config.'.$phpEx);

$startdate = date("Ymd-His");

// --------------------------------------------------------------------------
// Functions
// --------------------------------------------------------------------------

function open_db()
{
	global $db;
	global $dbhost;
	global $dbuser;
	global $dbpasswd;
	global $dbname;

	$db = mysql_connect($dbhost, $dbuser, $dbpasswd);
	mysql_select_db($dbname);
}

function close_db()
{
	global $db;
      mysql_close($db);
}

//
// remove_remarks will strip the sql comment lines out of the sql file
// (from phpbb2)
// extend with the check if a leading hashmark ("#") is part of a string
//
function remove_remarks($sql)
{
	$matches = array();

	$lines = explode("\n", $sql);

	// try to keep mem. use down
	$sql = "";

	$linecount = count($lines);
	$output = "";
    $total_quotes = 0;
    $is_in_string = false;

	for ($i = 0; $i < $linecount; $i++)
	{
		if (($i != ($linecount - 1)) || (strlen($lines[$i]) > 0))
		{


			if (($lines[$i][0] != "#") || ($is_in_string))
			{
				$output .= $lines[$i] . "\n";

			    // add the total number of single quotes in this line
			    $total_quotes_in_line = preg_match_all("/'/", $lines[$i], $matches);
			    // and subtract all single quotes that are preceded by an odd number
                // of backslashes, which means they're escaped quotes.
			    $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $lines[$i], $matches);

			    $total_quotes += $total_quotes_in_line - $escaped_quotes;

			    if (($total_quotes % 2) != 0){
				   // the following line is part of an open string.
                   // so a possible hashmark (#) at the first column do also.
				   $is_in_string = true;
			    }
                else {
                    $is_in_string = false;
                }
			}
			else
			{
				$output .= "\n";
			}

			// Trading a bit of speed for lower mem. use here.
			$lines[$i] = "";
		}
	}

	return $output;

}

//
// split_sql_file will split an uploaded sql file into single sql statements.
// Note: expects trim() to have already been run on $sql.
// (from phpbb2)
//
function split_sql_file($sql, $delimiter)
{
	// Split up our string into "possible" SQL statements.
	$tokens = explode($delimiter, $sql);

	// try to save mem.
	$sql = "";
	$output = array();

	// we don't actually care about the matches preg gives us.
	$matches = array();

	// this is faster than calling count($oktens) every time thru the loop.
	$token_count = count($tokens);
	for ($i = 0; $i < $token_count; $i++)
	{
		// Don't wanna add an empty string as the last thing in the array.
		if (($i != ($token_count - 1)) || (strlen($tokens[$i] > 0)))
		{
			// This is the total number of single quotes in the token.
			$total_quotes = preg_match_all("/'/", $tokens[$i], $matches);
			// Counts single quotes that are preceded by an odd number of backslashes,
			// which means they're escaped quotes.
			$escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$i], $matches);

			$unescaped_quotes = $total_quotes - $escaped_quotes;

			// If the number of unescaped quotes is even, then the delimiter did NOT occur inside a string literal.
			if (($unescaped_quotes % 2) == 0)
			{
				// It's a complete sql statement.
				$output[] = $tokens[$i];
				// save memory.
				$tokens[$i] = "";
			}
			else
			{
				// incomplete sql statement. keep adding tokens until we have a complete one.
				// $temp will hold what we have so far.
				$temp = $tokens[$i] . $delimiter;
				// save memory..
				$tokens[$i] = "";

				// Do we have a complete statement yet?
				$complete_stmt = false;

				for ($j = $i + 1; (!$complete_stmt && ($j < $token_count)); $j++)
				{
					// This is the total number of single quotes in the token.
					$total_quotes = preg_match_all("/'/", $tokens[$j], $matches);
					// Counts single quotes that are preceded by an odd number of backslashes,
					// which means they're escaped quotes.
					$escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$j], $matches);

					$unescaped_quotes = $total_quotes - $escaped_quotes;

					if (($unescaped_quotes % 2) == 1)
					{
						// odd number of unescaped quotes. In combination with the previous incomplete
						// statement(s), we now have a complete statement. (2 odds always make an even)
						$output[] = $temp . $tokens[$j];

						// save memory.
						$tokens[$j] = "";
						$temp = "";

						// exit the loop.
						$complete_stmt = true;
						// make sure the outer loop continues at the right point.
						$i = $j;
					}
					else
					{
						// even number of unescaped quotes. We still don't have a complete statement.
						// (1 odd and 1 even always make an odd)
						$temp .= $tokens[$j] . $delimiter;
						// save memory.
						$tokens[$j] = "";
					}

				} // for..
			} // else
		}
	}

	return $output;
}

// --------------------------------------------------------------------------
// End Functions
// --------------------------------------------------------------------------

// read some parameters and avoid unallowed html-code

if (isset($_GET['task'])) {
   $task=htmlspecialchars(strip_tags($_GET['task']));
}
else {
   $task="start";
}

if (isset($_GET['dnlfile'])) {
   $dnlfile=htmlspecialchars(strip_tags($_GET['dnlfile']));
}
else {
   $dnlfile="";
}

if (isset($_GET['rsfile'])) {
   $rsfile=htmlspecialchars(strip_tags($_GET['rsfile']));
}
else {
   $rsfile="";
}

if (isset($_GET['ack'])) {
   $ack=htmlspecialchars(strip_tags($_GET['ack']));
}
else {
   $ack="no";
}


if (!(($task=="start") || ($task=="download") || ($task=="restore"))) {
       echo 'wrong parameters <a href="restore_db.php">return</a>;';
}
       
switch ($task) {

   case "download":

         if ($dnlfile == "") {
             die ("Name of download file missing");
         }
         else {
             $fullname = $backupdir . '/' . $dnlfile;
             if (file_exists ($fullname) && is_file ($fullname) && is_readable ($fullname)) {
                header ('Pragma: no-cache');
                header("Content-Type: text/x-delimtext; name=\"" . $dldatei . "\"");
                header("Content-disposition: attachment; filename=" . $dldatei);
                $groesse = filesize($fullname);
                header("Content-Lenght:" . $groesse);
                $f = fopen($fullname, 'r');
                $i = 0;
                $s = 0;
                while ($i < $groesse) {
                    if (($i+100000) > $groesse) {
                        $s = $groesse - $i;
                    }
                    else {
                        $s=100000;
                    }
                    $i += $s;
                    $data = fread($f, $s);
                    echo $data;
                }
             }
             else {
                 die ("Error download file");
             }
         }

         break;
         
   case "start":

        if ($handle = opendir($backupdir)) {

           echo '<font face="Arial"><font size=3>Script <font color=blue>restore_db.php</font> started at ' . $startdate . ' --- (c) 2007 by Ron</font></font><br><br>';

           echo '<table border=1 valign=left cellspacing=2 cellpadding=1
                 bgcolor="#FFFFFF" bordercolorlight="#E0E0E0" bordercolordark="#989898">';
           echo '<tr><td valign=middle height=20><p align=center>
                         <font size=2><font face="Arial">File</font></font></p></td>
                     <td valign=middle height=20><p><font size=2>
                         <font face="Arial">Size</font></font></p></td>
                     <td valign=middle height=20><p><font size=2>
                         <font face="Arial">download</font></font></p></td>
                     <td valign=middle height=20><p><font size=2>
                         <font face="Arial">restore</font></font></p></td>
                 </tr>';

           while (false !== ($file = readdir($handle))) {
               $fullname = $backupdir . '/' . $file;
               if (file_exists ($fullname) && is_file ($fullname) && is_readable ($fullname)) {
                 echo '<tr><td valign=left>
                              <p align=left><font size=2><font face="Arial">' . $file .
                                 '</font></font></p></td>
                           <td valign=right>
                              <p align=right><font size=2><font face="Arial">' .
                              filesize($fullname) . '</font></font></p></td>
                           <td valign=left>
                              <p align=left><font size=2><font face="Arial">
                              <a href="restore_db.php?task=download&dnlfile=' . $file . '">download</a>
                              </font></font></p></td>
                           <td valign=left>
                              <p align=left><font size=2><font face="Arial">
                              <a href="restore_db.php?task=restore&rsfile=' . $file . '">restore</a>
                              </font></font></p></td>
                      </tr>';
               }
           }
           closedir($handle);
        }
        else {
           die ("Error reading directory");
        }
        
        break;

   case "restore":

        if ($ack=="yes") {

           $outbuffer = " ";
           for ($i=0; $i<4096; $i++) { $outbuffer .=" ";}

           echo 'Starting restore file >' . $rsfile . '< ... please wait ---------- ' .
                'click here to <a href="restore_db.php">return</a><br><br>';

           echo $outbuffer;
           flush();

           open_db();
           
           $backupfile = $backupdir . '/' . $rsfile;

           $sql_query = fread(fopen($backupfile, 'r'), filesize($backupfile));

           if ($sql_query != "") {
              // Strip out sql comments...
     		  $sql_query = remove_remarks($sql_query);

              $pieces = split_sql_file($sql_query, ";");

              $sql_count = count($pieces);

              $step = 1000; // each step an outout will be performed
              if ($sql_count<50000) $step=500;
              if ($sql_count<10000) $step=100;
              if ($sql_count<5000)  $step=50;
              if ($sql_count<1000)  $step=20;
              if ($sql_count<100)   $step=10;
              if ($sql_count<20)    $step=1;

              echo "$sql_count sql statements needs to be processed.<br>" .
                    "You will get an information line each $step statements. continuing ...$outbuffer<br><br>";
		      flush();


              $j=0;
              $anz_fehler=0;
              
              for ($i = 0; $i < $sql_count; $i++) {
                 $sql = trim($pieces[$i]);

                 if(!empty($sql) and $sql[0] != "#") {
                    //echo "<br>Executing: $sql\n<br>";
					//flush();

                    $result = mysql_query($sql);
					if (!$result) {
                       echo "<br><br><font color=red>Error processing statement > </font><font size=1><font color=blue>$sql</font></font><font color=red> < continuing ...</font><br><br>";
                       $anz_fehler++;
					}
                    else {
                       $j++;
                       $k = bcmod ($j, $step);
                       if (($i==($sql_count-1)) || (!$k)) {
                           $ratio = bcdiv($j * 10000 / $sql_count, 1) / 100;
                           echo "$j of $sql_count sql statements processed ($ratio%) continuing ...$outbuffer<br>";
                           flush();
                       }
                    }

                 }
              }
              $anz_richtig = $sql_count - $anz_fehler;
              echo "<br>$anz_richtig statements were sucessfully processed<br>" .
                   "$anz_fehler statements failured<br><br>";
		   }
           close_db();
           
           echo '<br>finished!<br><a href="restore_db.php">return</a>';

        }
        else {
           echo '<p align=left><font size=4><font face="Arial">' .
                'Are you really sure to restore the file<br><br><font color=red>' . $rsfile .
                '</font><br><br>into your database? <br><br>' .
                'All corresponding database tables will be overwritten!<br><br>' .
                'Your forum will might be reset to an old state!<br><br>' .
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' .
                '<a href="restore_db.php?task=restore&ack=yes&rsfile=' . $rsfile . '">YES</a>' .
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' .
                '<a href="restore_db.php">NO</a>' .
                '</font></font></p></td>';
        }
        
        break;
}
?>
